//
//  DetailViewController.m
//  FragmentShaderBase
//
//  Created by 神谷 栄治 on 12/05/24.
//  Copyright (c) 2012年 株式会社アイビス. All rights reserved.
//

#import "IBGLKit.h"
#import "DetailViewController.h"
#import "GLViewController.h"
#import "MyShader.h"
#import "Configuration.h"


#define TAG_BUILD_ERROR 100

//プライベート
@interface DetailViewController (){
	//キーボードのサイズを記録したか
	BOOL _isKeyboardSizeRecorded;
	//キーボードサイズ
	CGRect _keyboardRect;
}
- (void)configureView;
- (void)setInitialSource;
- (void)applyKeyboardSize:(NSNotification *)notif;
- (void)keyboardDidChange:(NSNotification *)notif;
- (void)keyboardWillShow:(NSNotification *)notif;
- (void)keyboardWillHide:(NSNotification *)notif;
- (void)setTextViewSize;
- (void)onTappedNextButton:(id)sender;
// マスターポップオーバー
@property(strong, nonatomic) UIPopoverController* masterPopoverController;
@end

@implementation DetailViewController

@synthesize masterPopoverController = _masterPopoverController;
@synthesize detailItem = _detailItem;
@synthesize textView = _textView;

//初期化
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
		self.title = NSLocalizedString(@"FshSource", nil);
		_isKeyboardSizeRecorded = NO;
    }
    return self;
}

//ビューロード後イベント
- (void)viewDidLoad
{
    [super viewDidLoad];
	// テキストビュー
	self.textView.autoresizingMask = UIViewAutoresizingFlexibleHeight;
	// 右上のNextボタン
	UIBarButtonItem *nextButton = [[UIBarButtonItem alloc] 
								   initWithTitle:@"Next" style:UIBarButtonItemStyleDone 
								   target:self action:@selector(onTappedNextButton:)];
	self.navigationItem.rightBarButtonItem = nextButton;
	self.navigationItem.rightBarButtonItem.enabled=NO;
	// キーボード表示通知を登録
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(keyboardWillShow:) 
												 name:UIKeyboardWillShowNotification 
											   object:self.view.window];
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(keyboardWillHide:) 
												 name:UIKeyboardWillHideNotification 
											   object:self.view.window];
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(keyboardWillShow:)
												 name:UIKeyboardDidChangeFrameNotification
											   object:self.view.window];
	[self configureView];
}

// ビューアンロードイベント
- (void)viewDidUnload
{
    [super viewDidUnload];
	// キーボード表示通知を解除
    [[NSNotificationCenter defaultCenter] removeObserver:self 
													name:UIKeyboardWillShowNotification 
												  object:self.view.window]; 
    [[NSNotificationCenter defaultCenter] removeObserver:self 
													name:UIKeyboardWillHideNotification 
												  object:self.view.window]; 
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:UIKeyboardDidChangeFrameNotification
												  object:self.view.window];
	// 解放
	self.textView = nil;
	self.detailItem = nil;
}

// ビュー表示前イベント
- (void)viewWillAppear:(BOOL)animated {
	self.textView.frame = self.view.frame;
}

// ビュー非表示前イベント
- (void)viewWillDisappear:(BOOL)animated {
	assert(self.detailItem);
	NSNumber* num = self.detailItem;
	[Configuration setFshSource:self.textView.text no:[num intValue]];
	[self.textView resignFirstResponder];	
}

// 回転していいか？
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
	if([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone){//iPhoneのとき
		return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
	} else {//iPadのとき
		return YES;
	}
}

//回転前イベント
-(void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation 
							   duration:(NSTimeInterval)duration{
	//キーボードのサイズを再度取得させる
	_isKeyboardSizeRecorded = NO;
}

// 回転後イベント
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)oriantation 
{
	[self setTextViewSize];
}

//テキストビューの大きさを設定
-(void)setTextViewSize{
	if(_isKeyboardSizeRecorded){
		CGFloat width = _keyboardRect.size.width;
		CGRect frame = CGRectMake(0.0,0.0, width,
								  self.view.frame.size.height - _keyboardRect.size.height);
		if(!(self.textView.frame.size.width==frame.size.width && 
			 self.textView.frame.size.height==frame.size.height)){//サイズが違うとき
			self.textView.frame = frame;
		}
	}
}

// マスター画面でアイテムが変更されたイベント
- (void)setDetailItem:(id)newDetailItem
{
	// 保存
	NSNumber* num = self.detailItem;
	[Configuration setFshSource:self.textView.text no:[num intValue]];
	// テキストビューの内容変更
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;        
        [self configureView];
    }
	if(self.masterPopoverController != nil){// iPad立て画面のとき
		[self.masterPopoverController dismissPopoverAnimated:YES];
	}
}

// 変更を更新する
- (void)configureView
{
	if (self.detailItem==nil) {
		self.textView.text = @"";
		self.textView.userInteractionEnabled=NO;
		self.navigationItem.rightBarButtonItem.enabled=NO;
		self.title = @"No Source";
		return;
	}
	NSNumber* num = self.detailItem;
	NSString* fshSrc = [Configuration fshSouce:[num intValue]];
	if(fshSrc && fshSrc.length > 0){//ソースがあるとき
		self.textView.text = fshSrc;
	} else {
		[self setInitialSource];
	}

	self.navigationItem.rightBarButtonItem.enabled=YES;
	self.textView.userInteractionEnabled=YES;
	self.title = [NSString stringWithFormat:@"FshSource%d",[num intValue]];
}

// テキストビューサイズをキーボードサイズに合わせて変化させる
-(void)applyKeyboardSize:(NSNotification *)notif{	
	CGRect viewFrame = self.view.bounds;
	
	//キーボードの位置とサイズを新しく取得
	NSValue *boundsValue = [notif.userInfo valueForKey:UIKeyboardFrameEndUserInfoKey];
	CGRect keyboardWinRect;
	[boundsValue getValue:&keyboardWinRect];
	CGRect windowRect = [self.view.window convertRect:keyboardWinRect fromWindow:nil];
	_keyboardRect = [self.view convertRect:windowRect fromView:nil];
	
	//キーボードとかぶらないよう、キーボードの高さ分小さくする
	viewFrame.size.height = self.view.bounds.size.height - _keyboardRect.size.height;
	self.textView.frame = viewFrame;
	_isKeyboardSizeRecorded = YES;
	[self setTextViewSize];
}

//キーボード表示前イベント
- (void)keyboardWillShow:(NSNotification *)notif
{	
	[self applyKeyboardSize:notif];
}

//キーボードサイズ変更イベント
-(void)keyboardDidChange:(NSNotification *)notif{
	[self applyKeyboardSize:notif];
}

//キーボード非表示前イベント
- (void)keyboardWillHide:(NSNotification *)notif{
	self.textView.frame = self.view.frame;
}

// 初期ソースを設定
-(void)setInitialSource
{
	NSString* src = @""
	"precision mediump float;\n"
	"varying vec2 v_texCoord;\n"
	"uniform lowp vec4 u_color;\n"
	"uniform lowp float u_alpha;\n"
	"uniform sampler2D u_texture;\n"
	"void main(){\n"
	"	gl_FragColor = texture2D(u_texture, v_texCoord);\n"
	"}\n";
	self.textView.text = src;
}

// Nextボタンタップイベント
-(void)onTappedNextButton:(id)sender
{
	// ビルド
	NSString* error = nil;
	@autoreleasepool { //glView即時破棄のために必要
		IBGLView* glView = [[IBGLView alloc] initWithCoder:nil];//buildにはGL contextが必要
		glView.tag = 0;//warning抑制
		MyShader* shader = [[MyShader alloc] init];
		shader.fragmentShader = self.textView.text;
		error = [shader build];
	}
	if(error==nil){//ビルド成功のとき
		[self.textView resignFirstResponder];//キーボード非表示
		GLViewController* glvc = [[GLViewController alloc] 
									initWithNibName:@"GLViewController" bundle:nil];
		[glvc setFragmentShaderSource:self.textView.text];
		[self.navigationController pushViewController:glvc animated:YES];
		return;
	}
	// エラーのとき
	UIAlertView* alert = [[UIAlertView alloc] 
						  initWithTitle:@"Error" message:error 
						  delegate:self 
						  cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
	alert.tag = TAG_BUILD_ERROR;
	[alert show];
	// 保存
	NSNumber* num = self.detailItem;
	[Configuration setFshSource:self.textView.text no:[num intValue]];
}

// アラートのボタンタップイベント
-(void)alertView:(UIAlertView*)alertView clickedButtonAtIndex:(NSInteger)idx {
	if(alertView.tag == TAG_BUILD_ERROR){//ビルドエラーのとき
	}
}

// マスタービューが消える前イベント
- (void)splitViewController:(UISplitViewController *)splitController 
	 willHideViewController:(UIViewController *)viewController 
		  withBarButtonItem:(UIBarButtonItem *)barButtonItem 
	   forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Fsh List", nil);
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

// マスタービューが表示される前イベント
- (void)splitViewController:(UISplitViewController *)splitController 
	 willShowViewController:(UIViewController *)viewController 
  invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

@end
