//
//  IBGLShaderPT.mm
//  FragmentShaderBase
//
//  Created by 神谷 栄治 on 12/05/12.
//  Copyright (c) 2012年 株式会社アイビス. All rights reserved.
//

#import "IBGLKit.h"
#import "IBGL.h"
#import "IBGLShaderPT.h"
#import "Matrix.h"

// プライベート
@interface IBGLShader(){
}
@end


@implementation IBGLShaderPT

enum VertexAttribute {
	VA_POSITION,
	VA_TEXCOORD,
};

enum UniformLocation {
	UL_MATRIX,
	UL_TEXTURE,
	UL_COLOR,
};

//ビルド
-(NSString*)build{
	NSString* vsh = @""
	"attribute vec2 a_position;\n"
	"attribute vec2 a_texCoord;\n"
	"uniform mat4 u_matrix;\n"
	"varying vec2 v_texCoord;\n"
	"void main(void){\n"
	"	gl_Position = u_matrix * vec4(a_position, 0.0, 1.0);\n"
	"	v_texCoord = a_texCoord;\n"
	"}\n";
	NSString* fsh = @"" 
	"precision mediump float;\n"
	"varying vec2 v_texCoord;\n"
	"uniform lowp vec4 u_color;\n"
	"uniform sampler2D u_texture;\n"
	"void main(){\n"
	"	gl_FragColor = texture2D(u_texture, v_texCoord) * u_color;\n"
	"}\n";
	NSArray* attribs = [NSArray arrayWithObjects:@"a_position", @"a_texCoord", nil];
	NSArray* uniforms = [NSArray arrayWithObjects:@"u_matrix", @"u_texture", @"u_color", nil];
	return [self buildWithVsh:vsh fsh:fsh attributeVariables:attribs uniformVariables:uniforms];
}

//描画
-(void)drawArraysPT:(uint)primitive positions:(float*)pos 
		 glImage:(IBGLImage*)image texCoords:(float*)texCoods count:(int)count color:(Color)color {
	[self use];
	glEnable(GL_BLEND);CHECK_GL_ERROR();
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);CHECK_GL_ERROR();
	glActiveTexture(GL_TEXTURE0);CHECK_GL_ERROR();//TextureUnit0
	[self setUniformInteger:0 at:UL_TEXTURE];//TextureUnit0
	glBindTexture(GL_TEXTURE_2D, image.textureId);CHECK_GL_ERROR();
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);	
	[self setVertexAttribute2f:pos at:VA_POSITION];
	[self setVertexAttribute2f:texCoods at:VA_TEXCOORD];
	[self setUniformColor:color at:UL_COLOR];
	[self setUniformMatrix:[IBGL sharedGL].viewMatrix at:UL_MATRIX];
	[self drawArraysNative:primitive count:count];
}
@end
